/// <reference types="node" />
import { EventEmitter } from 'events';
import { Task, RetryOptions } from './common';
import { BackendType } from './backends/interface';
import { MongodbBackendOptions } from './backends/mongodb';
import { BrokerType } from './brokers/interface';
import { AMQPBrokerOptions } from './brokers/amqp';
export interface ProducerOptions {
    /**
     * Test mode, useful for testing, default false
     */
    isTestMode?: boolean;
    /**
     *  Broker type, default: AMQP
     */
    brokerType?: BrokerType;
    /**
     *  Broker options
     */
    brokerOptions?: AMQPBrokerOptions;
    /**
     *  Backend type, default: MongoDB
     */
    backendType?: BackendType;
    /**
     *  Backend options
     */
    backendOptions?: MongodbBackendOptions;
    /**
     * Global retry options
     * Default:
     *     maxRetry: 0
     *     initDelayMs: 100
     *     delayMs: 100
     *     retryStrategy: FIBONACCI
     */
    globalRetryOptions?: RetryOptions;
}
export declare class Producer extends EventEmitter {
    private options;
    private backend;
    private broker;
    createdTasks: Task[];
    constructor(options?: ProducerOptions);
    createTask(task: Task): Promise<any>;
    close(): Promise<any>;
    checkHealth(): Promise<any>;
}
