"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var debug = require("debug");
var uuid = require("uuid");
var Bluebird = require("bluebird");
var common_1 = require("./common");
var interface_1 = require("./backends/interface");
var mongodb_1 = require("./backends/mongodb");
var interface_2 = require("./brokers/interface");
var amqp_1 = require("./brokers/amqp");
var helper_1 = require("./helper");
var log = debug('blackfyre:producer');
Promise = Bluebird;
var Producer = /** @class */ (function (_super) {
    __extends(Producer, _super);
    function Producer(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.createdTasks = [];
        _this.options = Object.assign({
            isTestMode: false,
            backendType: interface_1.BackendType.MongoDB,
            backendOptions: null,
            brokerType: interface_2.BrokerType.AMQP,
            brokerOptions: null,
        }, options);
        _this.options.globalRetryOptions = Object.assign({
            maxRetry: 0,
            initDelayMs: 100,
            delayMs: 100,
            retryStrategy: common_1.RetryStrategy.FIBONACCI,
        }, options.globalRetryOptions);
        log('Init backend %s', _this.options.backendType);
        if (_this.options.backendType === interface_1.BackendType.MongoDB) {
            _this.backend = new mongodb_1.MongodbBackend(_this.options.backendOptions);
            helper_1.registerEvent(['error', 'close'], _this.backend, _this);
        }
        log('Init broker %s', _this.options.brokerType);
        if (_this.options.brokerType === interface_2.BrokerType.AMQP) {
            _this.broker = new amqp_1.AMQPBroker(_this.options.brokerOptions);
            helper_1.registerEvent(['error', 'ready', 'close'], _this.broker, _this);
        }
        return _this;
    }
    Producer.prototype.createTask = function (task) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        task.id = task.id || uuid.v4();
                        task.retryCount = 0;
                        task = Object.assign({}, this.options.globalRetryOptions, task);
                        if (this.options.isTestMode) {
                            this.createdTasks.push(task);
                            return [2 /*return*/];
                        }
                        return [4 /*yield*/, this.backend];
                    case 1:
                        (_a.sent()) && this.backend.setTaskStatePending(task);
                        return [2 /*return*/, this.broker.publish(task)];
                }
            });
        });
    };
    Producer.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, Promise.props({
                        broker: this.broker.close(),
                        backend: this.backend && this.backend.close(),
                    })];
            });
        });
    };
    Producer.prototype.checkHealth = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, Promise.props({
                        backend: this.backend && this.backend.checkHealth(),
                        broker: this.broker.checkHealth(),
                    })];
            });
        });
    };
    return Producer;
}(events_1.EventEmitter));
exports.Producer = Producer;
//# sourceMappingURL=producer.js.map